@@ -91,7 +91,7 @@ QueueView::QueueView(Spotify::Session* pSession, OssoWrapper* pOsso, DBusWrapper
     connect(&m_NextBtn,SIGNAL(clicked()),SLOT(OnNextTrackBtn()));
     connect(menuBar(),SIGNAL(triggered(QAction*)),SLOT(OnMenubarAction(QAction*)));
     connect(QApplication::desktop(), SIGNAL(resized(int)),SLOT(OnOrientationChanged()));
-   // connect(m_pProgressSlider,SIGNAL(positionChanged(int)),SLOT(OnProgressSliderClicked(int)));
+    connect(m_pProgressSlider,SIGNAL(positionChanged(int)),SLOT(OnProgressSliderClicked(int)));
 
     m_AlbumCover.setFixedSize(QUEUEVIEW_COVERART_WIDTH_LANDSCAPE,QUEUEVIEW_COVERART_HEIGHT_LANDSCAPE); //set fixed size of the album coverart
 
@@ -653,9 +653,8 @@ void QueueView::OnPauseResumeToggle()
         }
         if(m_tokenLost)
         {//token has been lost; send request to regain the playback token
-           err =  m_pSession->pauseResumeBuffering();
-           if(!err.isError())
-               m_tokenLost = false;
+           m_pSession->pauseResumeBuffering();
+           m_tokenLost = false;
         }
         m_pSession->pauseResumePlayback(); //pause / resume playback of current track
     }
@@ -1219,11 +1218,10 @@ void QueueView::setHideCoverart(bool hide)
         m_AlbumCover.setVisible(true);
 }
 
-/*void QueueView::OnProgressSliderClicked(int pos)
+void QueueView::OnProgressSliderClicked(int pos)
 {
     //progress slider has been clicked (pos=clicked position)
     double seekPos;
-
     if(m_pCurrTrack)
     {
         //we have a track loaded, calculate corresponding position within the track (in msecs)
@@ -1240,5 +1238,4 @@ void QueueView::setHideCoverart(bool hide)
         m_pProgressSlider->setPos2(pos);
     }
 }
-*/
 
