@@ -82,8 +82,6 @@ private:
 
     QGridLayout m_LoginOptionLayout;
 
-    QCheckBox m_SaveCredentials; //checkbox to indicate whether the user login credentails should be stored (i.e., for auto-login)
-
     QCheckBox m_HideCoverArt; //checkbox to indicate whether the coverart should be hidden in the Queue View (to free more space for track info).
 
     QStringList m_bitrateList; //list of available bitrate alternatives
@@ -179,14 +177,8 @@ private:
 
     qint32 m_preBufferingActive;
 
-    qint32 m_saveCredentials;
-
     qint32 m_hideCoverart;
 
-    QString m_strUserID;
-
-    QString m_strUserPwd;
-
     qint32 m_prefBitrate;
 
     qint32 m_currOrientation;
