@@ -18,13 +18,23 @@ SpectrumAnalyzer::SpectrumAnalyzer(QWidget *parent, int width, int height)
 	this->setFixedSize(m_width, m_height);
 	m_color = QColor(0, 0, 0);
 
-	QTimer *timer = new QTimer(this);
-	connect(timer, SIGNAL(timeout()), this, SLOT(updateAnalyzers()));
-	timer->start(50);}
+        m_pTimer = new QTimer(this);
+        connect(m_pTimer, SIGNAL(timeout()), this, SLOT(updateAnalyzers()));
+ }
+
+void SpectrumAnalyzer::Start()
+{
+    m_playing=true;
+    m_pTimer->start(100);
+}
+
+void SpectrumAnalyzer::Stop()
+{
+    m_playing=false;
+}
 
 SpectrumAnalyzer::~SpectrumAnalyzer()
 {
-
 }
 
 void SpectrumAnalyzer::paintEvent(QPaintEvent *)
@@ -77,6 +87,7 @@ void SpectrumAnalyzer::updateAnalyzers()
 		m_p2 = m_height-1;
 		m_p3 = m_height-1;
 		m_p4 = m_height-1;
+                m_pTimer->stop();
 	}
 
 	update();
