@@ -5,8 +5,9 @@
 
 namespace Spotify
 {
-    Playlist::Playlist( sp_playlist* playlist, QObject* parent ):
+    Playlist::Playlist(sp_session* session, sp_playlist* playlist, QObject* parent ):
             TrackCollection( parent ),
+            m_session( session ),
             m_playlist( playlist ),
             m_owner( 0 ),
             m_position( 0 )
@@ -43,7 +44,7 @@ namespace Spotify
                 for( int i = 0; i < m_numTracks; ++i )
                 {
                     sp_track* track = sp_playlist_track( m_playlist, i );
-                    Spotify::Track* strack = new Spotify::Track( track, false, this );
+                    Spotify::Track* strack = new Spotify::Track(m_session, track, false, this );
                     m_tracks.append( strack );
                     m_tracksOriginal.append( strack );
                 }
