@@ -70,18 +70,19 @@ Spotify::Session::Session( QObject* parent ):
     m_bPlaylistsLoading = false;
 }
 
-QString Spotify::Session::getUserID()
+QString Spotify::Session::getRememberedUser()
 {
-    //Returns current UserID for the session
+    //Returns remembered user (in case logged in via sp_session_relogin)
     return m_username;
 }
 
 QString Spotify::Session::getUserPwd()
 {
-    //Returns current password for the session
+    //Returns remembered password for the session
     return m_password;
 }
 
+
 void Spotify::Session::init(const sp_bitrate& bitrate)
 {
     //Init the current Spotify session.
@@ -89,15 +90,15 @@ void Spotify::Session::init(const sp_bitrate& bitrate)
 
     // Set config:
     m_config.api_version = SPOTIFY_API_VERSION;
-    m_config.cache_location = "";
-    m_config.settings_location = "";
+    m_config.cache_location = "/home/user/MyDocs/.qspot";
+    m_config.settings_location = "/home/user/MyDocs/.qspot";
     m_config.application_key = g_appkey;
     m_config.application_key_size = g_appkey_size;
     m_config.user_agent = "QSpot";
     m_config.callbacks = &g_callbacks;
 
     // Initiate session:
-    m_error = sp_session_init( &m_config, &m_session );
+    m_error = sp_session_create( &m_config, &m_session );
     if( m_error.isError() )
     {
         qDebug() << "FATAL ERROR: Could not initiate session";
@@ -112,6 +113,12 @@ void Spotify::Session::init(const sp_bitrate& bitrate)
    connect(m_pProcEvtTimer,SIGNAL(timeout()),this,SLOT(OnProcessThreadEvents()));
 }
 
+void Spotify::Session::setPreferredBitrate(const sp_bitrate& bitrate)
+{
+    //set preferred audio playback bitrate
+     sp_session_preferred_bitrate(m_session, bitrate);
+}
+
 void Spotify::Session::initAudio()
 {
     //Init the audio / playback thread (pulseaudio).
@@ -244,42 +251,27 @@ void Spotify::Session::OnProcessThreadEvents()
    }
 }
 
-Spotify::Error Spotify::Session::login( QString username, QString password )
+void Spotify::Session::login( QString username, QString password, bool storeCredentials )
 {
-    // Log user in
+    // Request to login to Spotify with supplied credentials
 
     m_DisableNotifyThread = false;
 
-    m_error = sp_session_login( m_session, username.toUtf8(), password.toUtf8() );
-    if( m_error.isError() )
-    {
-        qDebug()<<"Login error";
-        m_valid = false;
-    }
-    else
-    { //successful login, store credentials temporarily during session.
-        m_username = QString(username);
-        m_password = QString(password);
-    }
-
-    return m_error;
+    sp_session_login( m_session, username.toUtf8(), password.toUtf8(), storeCredentials );
+  //successful login, store credentials temporarily during session.
+    m_username = QString(username);
+    m_password = QString(password);
 }
 
-Spotify::Error Spotify::Session::reconnect()
+Spotify::Error Spotify::Session::relogin()
 {
-    //Reconnects / re-logins current user to Spotify.
+    //Reconnects / re-logins remembered user to Spotify.
     //Used in case network connection etc. is temporarily lost, and reconnect is required.
+    //Returns SP_ERROR_OK on success; otherwise error code is set.
 
     m_buffering = false; //no buffering as we are not logged in
-
-    m_error = sp_session_login( m_session, m_username.toUtf8(), m_password.toUtf8() );
-    if( m_error.isError() )
-    {
-        qDebug()<<"Login error (reconnect)";
-        m_valid = false;
-    }
-
-    return m_error;
+    login(m_username,m_password);
+    return Spotify::Error(SP_ERROR_OK);
 }
 
 void Spotify::Session::logout()
@@ -340,11 +332,15 @@ void Spotify::Session::resetPlaybackMembers()
 Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
 {
     m_buffering = false;
-    m_currentTrack = track;
     Spotify::Error error;
 
-    //sleep.wait(&mutex,1300);
-	
+    if(m_currentTrack)
+    {//if track loaded; unload it first
+        sp_session_player_unload(m_session);
+        m_currentTrack = NULL;
+    }
+
+
     // Disregard tracks that are not available:
     if( !track->isAvailable() )
     {
@@ -357,7 +353,7 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
     if( !track->isLoaded() )
     {
         qDebug() << "ERROR: Asked to play back an unloaded track..";
-        return Spotify::Error( SP_ERROR_RESOURCE_NOT_LOADED );
+        return Spotify::Error( SP_ERROR_TRACK_NOT_PLAYABLE );
     }
 
     // Load track:
@@ -368,6 +364,8 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
         return error;
     }
 
+    m_currentTrack = track;
+
     if(!preBuffer) //pre-buffering not active (i.e., audio FIFO should be cleared)
     {
         if(m_pAudioThread)
@@ -385,28 +383,19 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
     }
 
     // Play track:
-    qDebug() << "+play";
-    error = sp_session_player_play( m_session, true );
-    qDebug() << "-play";
-    if( !error.isError())
+    sp_session_player_play( m_session, true );
+
+    if(!preBuffer) //track pre-buffer not active
     {
-        if(!preBuffer) //track pre-buffer not active
-        {
-            m_playbackPos = 0;
-            m_bufferedAmount = 0;
-        }
-        m_buffering = true;
-    }
-    else
-    {
-        qDebug() << "ERROR: Could not play track: " << error.toString();
-        return error;
+        m_playbackPos = 0;
+        m_bufferedAmount = 0;
     }
+    m_buffering = true;
 
     return Spotify::Error( SP_ERROR_OK );
 }
 
-Spotify::Error Spotify::Session::seek( qint32 pos )
+void Spotify::Session::seek( qint32 pos )
 {
     //Seek to specified position (pos) within the currenly loaded track. Pos is specified in msecs.
 
@@ -418,25 +407,19 @@ Spotify::Error Spotify::Session::seek( qint32 pos )
     m_pAudioThread->setFIFOSize(0);
     m_pAudioThread->getMutex()->unlock();
 
-    Spotify::Error error = sp_session_player_seek( m_session, pos );
-    return error;
+    sp_session_player_seek( m_session, pos );
  }
 
-Spotify::Error Spotify::Session::stop()
+void Spotify::Session::stop()
 {
-    //stop playback (buffering)
-    Spotify::Error error;
+    //stop buffering
 
     if( m_buffering )
     {
-        error = sp_session_player_play( m_session, false );
-        if( !error.isError() )
-        {
-           // sp_session_player_unload( m_session );
-            m_buffering = false;
-        }
+        sp_session_player_play( m_session, false );
+        sp_session_player_unload( m_session );
+        m_buffering = false;
     }
-    return error;
 }
 
 void Spotify::Session::pauseResumePlayback()
@@ -450,20 +433,15 @@ void Spotify::Session::pauseResumePlayback()
     m_pAudioThread->getMutex()->unlock();
 }
 
-Spotify::Error Spotify::Session::pauseResumeBuffering()
+void Spotify::Session::pauseResumeBuffering()
 {
     //Pause / resume  music delivery (i.e., buffering) from  Spotify
 
-    Spotify::Error error;
-
     //playing (buffering) music data
     if( m_buffering )
     { //pause buffering
-        error = sp_session_player_play( m_session, false );
-        if( !error.isError() )
-        {
-            m_buffering = false;
-        }
+        sp_session_player_play( m_session, false );
+        m_buffering = false;
     }
     // buffering currently paused
     else if( !m_buffering)
@@ -475,16 +453,12 @@ Spotify::Error Spotify::Session::pauseResumeBuffering()
                 if(sp_track_is_loaded( m_currentTrack->getTrack()))
                 {
                     //resume buffering
-                    error = sp_session_player_play( m_session, true );
-                    if( !error.isError() )
-                    {
-                         m_buffering = true;
-                    }
+                    sp_session_player_play( m_session, true );
+                    m_buffering = true;
                 }
             }
         }
     }
-    return error;
 }
 
 Spotify::Error Spotify::Session::search( const QString& query, const int trackOffset, const int trackCount,
@@ -496,14 +470,14 @@ Spotify::Error Spotify::Session::search( const QString& query, const int trackOf
                                           albumOffset, albumCount, artistOffset, artistCount,
                                           &searchCompleteCallback, userdata );
 
-    Spotify::Search* s = new Spotify::Search( search, this );
+    Spotify::Search* s = new Spotify::Search(m_session, search, this );
     m_searchHash.insert( search, s );
     connect( s, SIGNAL(destroyed(QObject*)),
              this, SLOT(searchDeletedSlot(QObject*)));
     return error;
 }
 
-Spotify::Error Spotify::Session::loadImage(const byte* imageID, Spotify::ImageContainer* pContainer)
+bool Spotify::Session::loadImage(const byte* imageID, Spotify::ImageContainer* pContainer)
 {
     //request to load image (identified by imageID) from the Spotify backend.
 
@@ -511,13 +485,12 @@ Spotify::Error Spotify::Session::loadImage(const byte* imageID, Spotify::ImageCo
     if( imageID && pContainer )
     {
         sp_image* image = sp_image_create( m_session, imageID );
-        error = sp_image_error( image );
         if(image)
             sp_image_add_load_callback( image, &Spotify::Session::imageLoadedCallback, (void*)pContainer );
+        return true;
     }
-    else
-        error = SP_ERROR_RESOURCE_NOT_LOADED;
-    return error;
+
+    return false;
 }
 
 void Spotify::Session::getPlaylistContainer()
@@ -534,7 +507,7 @@ void Spotify::Session::getPlaylistContainer()
              NULL );
 
     // Go fetch container
-  sp_session_playlistcontainer( m_session );
+    sp_session_playlistcontainer( m_session );
 }
 
 
@@ -571,15 +544,8 @@ Spotify::Error Spotify::Session::getPlaylists()
 	  qDebug() << "Adding playlist: " << sp_playlist_name(pl);
 	  playlist->load(); //playlist loaded, load metadata
 	}
+  }
 
-  /* if(i>=m_playlistList.size())
-   {
-       qDebug() << "Adding playlist: " << sp_playlist_name(pl);
-       Spotify::Playlist* playlist = new Spotify::Playlist(pl,this);
-       m_playlistList.insert(i,playlist); //we could add the following element to the list elements in case of unsupported commands
-   }
-   */
-}
   m_bPlaylistsLoading = false; //indicate that playlist loading done (successfully)
 
   emit playlistContainersReady(m_playlistList);
@@ -595,45 +561,6 @@ void Spotify::Session::stopPlaylistLoading()
     m_bStopPlaylistLoading = true;
 }
 
-Spotify::Error Spotify::Session::getPlaylistTracks(Spotify::Playlist* pPlaylist)
-{
-	//get / update tracks for the specified playlist
-      QWaitCondition sleep;
-      QMutex mutex;
-
-      m_ProcessEvents = false;
-      m_DisableNotifyThread = true;
-
-      int next_timeout = -1;
-
-      for(int trackPos = 0; trackPos < pPlaylist->getNumTracks(); trackPos++)
-      {
-        Spotify::Track* track = pPlaylist->getTrack(trackPos);
-        if(track)
-        { 
-         /* while(!sp_track_is_loaded(track->getTrack()))
-          { //loop and process events until track is loaded
-            qDebug()<< "Attempting to load unloaded track..";
-            sp_session_process_events(m_session,&next_timeout);
-            qApp->processEvents();
-            sleep.wait(&mutex,100); //sleep for 100ms
-          }
-          */
-          track->load();
-          if(track->isAvailable() && track->getAlbum())
-          {
-            //attempt to load coverart for the track album
-            qDebug()<<"Requesting coverart..";
-            //getAlbumCover(track->getAlbum());
-          }
-	}
-      }
-
-      emit playlistTracksReady(pPlaylist); //notify track data is ready for the playlist
-
-      m_DisableNotifyThread = false;
-}
-
 bool Spotify::Session::addPlaylist( QString name )
 {
     if(!m_playlistcontainer) return false;
@@ -665,17 +592,16 @@ bool Spotify::Session::removePlaylist( const int index )
 
 bool Spotify::Session::movePlaylist( const int index, int position )
 {
-    if( !m_playlistcontainer) return false;
-    if( position > index ) position+=1; // libspotify wants from n to n+2 rather than n to n+1.....
-    Spotify::Error error = sp_playlistcontainer_move_playlist( m_playlistcontainer, index, position );
-    return !error.isError();
+    //if( !m_playlistcontainer) return false;
+    //if( position > index ) position+=1; // libspotify wants from n to n+2 rather than n to n+1.....
+    //Spotify::Error error = sp_playlistcontainer_move_playlist( m_playlistcontainer, index, position );
+    return false;
 }
 
 void Spotify::Session::updatePlaylistsMetadataSlot()
 {
     qDebug() << "metadata updated..";
 
-    bool allPlaylistsLoaded = true;
 
     for( int i = 0; i < m_playlistList.count(); ++i )
     {
@@ -684,19 +610,9 @@ void Spotify::Session::updatePlaylistsMetadataSlot()
         {
             playlist->load();
             playlist->updateTracks();
-            if(!playlist->isLoaded())
-            {
-                allPlaylistsLoaded = false;
-            }
-            else
-            {
-                qDebug() << "Loaded playlist: " << playlist->getName();
-            }
         }
     }
 
-    if(allPlaylistsLoaded)
-        emit playlistContainersReady(m_playlistList);
 
     // Try to load searches:
     QList<Spotify::Search*> searches = m_searchHash.values();
@@ -767,7 +683,7 @@ bool Spotify::Session::browse( Spotify::Album* album, qint32* pCallerID )
             return false; //failed to create browse album request
 
         //error = sp_albumbrowse_error( ab );
-        Spotify::AlbumBrowser* alb = new Spotify::AlbumBrowser( ab, this );
+        Spotify::AlbumBrowser* alb = new Spotify::AlbumBrowser(m_session, ab, this );
         m_albumBrowseHash.insert( ab, alb);
         connect( alb, SIGNAL(destroyed(QObject*)),
                  this, SLOT(albumBrowserDeletedSlot(QObject*)));
@@ -789,7 +705,7 @@ void Spotify::Session::albumBrowserDeletedSlot( QObject* object )
 }
 
 Spotify::Session::~Session()
-{
+{ //free allocated resources
     m_ProcessEvents = false;
     if(m_currentUser)
         m_currentUser->deleteLater();
@@ -798,6 +714,7 @@ Spotify::Session::~Session()
         m_pAudioThread->stop();
         m_pAudioThread->wait(); //wait for thread execution to finish
     }
+    sp_session_release(m_session); //free current session
     qDebug() << "Session deleted";
 }
 
@@ -873,7 +790,7 @@ void Spotify::Session::OnPlayListAdded(sp_playlist* playlist, int position)
 
    qDebug()<<"Playlist added: " << sp_playlist_name(playlist);
 
-   Spotify::Playlist* pl = new Spotify::Playlist( playlist );
+   Spotify::Playlist* pl = new Spotify::Playlist(m_session, playlist );
     pl->setParent( this );
    pl->setPosition(position);
    m_playlistHash.insert( playlist, pl );
@@ -979,17 +896,16 @@ void Spotify::Session::OnRootListContainerLoaded(sp_playlistcontainer* pc)
      m_playlistcontainer = pc;
 
      m_playlistList.clear(); //clear list of playlists
-	 
-	 //add initially empty playlist container instances
-	 for(qint32 pos = 0; pos < sp_playlistcontainer_num_playlists( pc ); pos++)
-	 {
-		 Spotify::Playlist* playlist = new Spotify::Playlist(sp_playlistcontainer_playlist(pc, pos), this);
+     //add initially empty playlist container instances
+     for(qint32 pos = 0; pos < sp_playlistcontainer_num_playlists( pc ); pos++)
+        {
+                 Spotify::Playlist* playlist = new Spotify::Playlist(m_session, sp_playlistcontainer_playlist(pc, pos), this);
 		 m_playlistList.insert(pos, playlist); //insert container instance into list
 	 }
 
          //playlist metadata will be loaded one we receive corresponding metadata updated callbacks from backend
 
-     //getPlaylists(); //retrieve actual playlist container metadata
+     getPlaylists(); //retrieve actual playlist container metadata
 }
 
 void Spotify::Session::tracksAddedCallback( sp_playlist* pl, sp_track* const* tracks, int num_tracks,
@@ -1021,7 +937,7 @@ void Spotify::Session::OnTracksAdded(sp_playlist* pl, sp_track* const* tracks, i
                 sp_track* tr = const_cast< sp_track* >( tracks[i] );
                 if( tr )
                 {
-                    Spotify::Track* track = new Spotify::Track( tr );
+                    Spotify::Track* track = new Spotify::Track(m_session, tr );
                     track->setParent( playlist );
                     trackList.append( track );
                 }
@@ -1387,13 +1303,7 @@ void Spotify::Session::OnMusicDelivery(int num_frames, const sp_audioformat* for
 
     emit trackBufferingAmountUpdated( m_bufferedAmount );
 
-    //split audio data buffer into max 1 sec chunks
-
-    for(int bufPos = 0; bufPos < buffer.size(); bufPos+=format->sample_rate*sizeof(int16_t)*format->channels)
-    {
-        bufPart = buffer.mid(bufPos,format->sample_rate*sizeof(int16_t)*format->channels);
-        m_pAudioThread->FIFOAppend(bufPart);
-    }
+    m_pAudioThread->FIFOAppend(buffer);
 
     m_pAudioThread->incFIFOSize(buffer.size());
 
@@ -1449,6 +1359,11 @@ void Spotify::Session::endOfTrackCallback( sp_session* session )
 void Spotify::Session::endOfTrackWrapper()
 {
     qDebug()<<"End of track callback..";
+    if(m_currentTrack)
+    { //unload current track
+        // sp_session_player_unload(m_session);
+        //m_currentTrack = NULL;
+    }
     if(!m_preBufActive)
     {
         m_pAudioThread->getMutex()->lock();
